import * as React from 'react';
import WaveSurfer from 'wavesurfer.js';
import {AudioFile, getMimeTypeDescription} from '../../utils/blob';
import {formatFileSize, formatNumeral, formatSecondsDurationPrecise} from '../../utils/numeral';

export interface VideoFilePreviewProps {
  file: AudioFile;
}

export const AudioMiniFilePreview: React.FC<VideoFilePreviewProps> = ({
  file: f,
}) => {
  const mediaContainerRef = React.useRef<HTMLDivElement>(null);
  const mediaControllerRef = React.useRef<WaveSurfer>(null);
  const [isPlaying, setIsPlaying] = React.useState(false);

  React.useEffect(() => {
    if (!mediaControllerRef.current) {
      return;
    }

    if (isPlaying) {
      void mediaControllerRef.current.play();
      return;
    }

    mediaControllerRef.current.pause();
  }, [isPlaying]);

  React.useEffect(() => {
    if (!mediaControllerRef.current) {
      return;
    }

    if (!mediaContainerRef.current) {
      return;
    }

    if (typeof f.metadata?.previewUrl !== 'string') {
      return;
    }

    mediaContainerRef.current.innerHTML = '';
    const mediaControllerRefMutable = mediaControllerRef as React.MutableRefObject<WaveSurfer>;
    mediaControllerRefMutable.current = WaveSurfer.create({
      container: mediaContainerRef.current,
      url: f.metadata.previewUrl,
      cursorWidth: 0,
      height: mediaContainerRef.current.offsetHeight,
      barWidth: 2,
      barGap: 2,
      barRadius: 1,
    });

    mediaControllerRefMutable.current.on('finish', () => {
      setIsPlaying(false);
      mediaControllerRefMutable.current.seekTo(0);
    });

    return () => {
      if (!mediaControllerRefMutable.current) {
        return;
      }

      mediaControllerRefMutable.current.destroy();
    }
  }, [f, mediaContainerRef, mediaControllerRef]);

  const playMedia = () => {
    setIsPlaying((p) => !p);
  };

  return (
    <div
      className="absolute top-0 left-0 w-full h-full cursor-pointer"
      ref={mediaContainerRef}
      onClick={playMedia}
    />
  );
};
